%% fit_INST_ARL2_part1.m
%
% Fit the INST model (aka protoanchor) to the group ARL2 profiles of
% the Fdbk1 Experiment.  The goal is to show that this model *cannot*
% fit these data very well.
%
% The main tool is .../work/models/protoanchor/protoanchor_ARL2.m
%
%  File: work/CMUExper/fdbk1/data/fit_INST_ARL2_part1.m
%  Date: 2007-08-16
%  Alexander Petrov, http://alexpetrov.com


%% Load the data
clear all
cd(fullfile(work_pathstr,'CMUExper','fdbk1','data')) ;
load('S2.mat') ;
ARL2 = [S2(:).ARL2] ;    % 9x55
gr=[S2(:).group]' ;
ugr = unique(gr) ;
N_groups = length(ugr) ;
ugr_descr = {'U1' 'L1' 'H1', 'L2' 'H2' } ;   % [1 2 3, 5 6]

%% Collapse across groups
grARL2 = zeros(size(ARL2,1),N_groups) ;   % 9x5
gr_idx = cell(1,N_groups) ;
for k = 1:N_groups
    gr_idx{k} = find(gr==ugr(k))' ;
    grARL2(:,k) = mean(ARL2(:,gr_idx{k}),2) ;
end
grARL2

%% Fit INST on the same stimulus sequences, 11 per group
stim = [S2(:).dist] ;   % 476x55
fdbk = [S2(:).fdbk] ;

%% Search parameters
% These control the optimizer via the PARAMSEARCH Toolbox.
% FMINCON is used (instead of LSQNONLIN) because experimental fits of
% synthetic ARL2 profiles showed that it works somewhat better in our
% setting.  Still, the cutoff parameters cannot be fitted with these
% methods, probably because the error derivative is not continuous w.r.t.
% the cutoffs.  The plan is to explore the two cutoff parameters on a
% grid, while optimising w.r.t three others:  mem_k, temper, and hist.
% The perceptual noise is fixed a priori to perc_k=.040.
% The history weight parameter is poorly determined by the data.
% Therefore, at first we fix it at H=.040 and do a grid search on the
% cutoff parameters. When we narrow down the relevant region of the
% parameter space, we will revisit H again.
fminconp = 1 ;
flags = [0 1 1 0 0 0 0 0] ;   % [perc_k mem_k temper hist cutoff xx xx c_ratio]
Sparams = protoanchor_search_params(flags,fminconp) ;
optns = optimset(Sparams.optns,'TolX',1e-2,'Display','off') ;
Sparams.optns = optns ;
Sparams.target_vals = grARL2
H = .040 ; Sparams.params.history = H ;   % for the time being
Sparams.params

%% "Chance event" sets
% The model is fitted using the deterministic version PROTOANCHOR_CHEV
% that uses previously cached values of all randomly generated values.
% More than one such "chev" sets is used to evaluate its impact on the fit.
N_chevs = 2 ;

% Pack into "model data" structures suitable for passing as arguments.
Mdata = cell(1,N_chevs) ;
for k = 1:N_chevs
    Mdata{k}.stim = stim ;
    Mdata{k}.fdbk = fdbk ;
    Mdata{k}.group_idx = gr_idx ;
    %Mdata{k}.params = Sparams.params ;    % will be set by SUMSQERR
    Mdata{k}.chev = make_protoanchor_chev(size(stim,1),size(stim,2)) ;
    Mdata{k}.ARL_params = Sparams.ARL_params ;
end

%% Cutoff parameter grid
cutoff_levels = [.50 .55 .60 .65 .70 .80 .90 1.00]' ;  % c- in categ_sz units
N_cutoff_levels = length(cutoff_levels) ;
c_ratio_levels = [.5 .6 .7 .8 .9] ;          % c-/c+ ratio
N_c_ratio_levels = length(c_ratio_levels) ;

%% Main work
% A grid search over 5x3x2 cells takes five hours on a fast machine.
% Therefore, the results will only be computed once and cached to disk.
fname = 'grid_search_INST_ARL2_part1.mat' ;
if (file_exists(fname))
    load(fname) ;
else
  grid_SSE = zeros([N_cutoff_levels,N_c_ratio_levels,N_chevs]) ;
  grid_optX = cell([N_cutoff_levels,N_c_ratio_levels,N_chevs]) ;
  grid_det = cell([N_cutoff_levels,N_c_ratio_levels,N_chevs]) ;
  tic
  for k1 = 1:N_cutoff_levels
      c = cutoff_levels(k1) ;
      fprintf('\n cutoff %4.2f: ',c) ;
      for k2 = 1:N_c_ratio_levels
          cr = c_ratio_levels(k2) ;
          cutoffs = c .* [-3 -1 cr 3*cr] ;
          Sparams.params.cutoffs = cutoffs ;
          for k = 1:N_chevs
              [par,SSE,optX,det] = paramsearch(Mdata{k},Sparams) ;  % <-- Sic!
              grid_SSE(k1,k2,k) = SSE ;
              grid_optX{k1,k2,k} = optX ;
              grid_det{k1,k2,k} = det ;
              fprintf('%5.3f ',SSE) ;
              save(fname,'grid_SSE','grid_optX','grid_det','Sparams',...
                   'cutoff_levels','c_ratio_levels') ;
          end
          fprintf(', ') ;
      end
  end
  fprintf('\n') ;
  toc
end % if file_exists ...

%% Plot the SSE
SSE = mean(grid_SSE,3)   % average across the chev's
[X,Y] = meshgrid(c_ratio_levels,cutoff_levels) ; 
contourf(X,Y,SSE) ; colorbar ; title('SSE') ;
xlabel('c-/c+ ratio') ; ylabel('c- cutoff') ;

%% Plot the memory noise parameter
mem_k = zeros(size(grid_optX)) ;
temper = zeros(size(grid_optX)) ;
for k1 = 1:N_cutoff_levels
    for k2 = 1:N_c_ratio_levels
        for k = 1:N_chevs
            mem_k(k1,k2,k) = grid_optX{k1,k2,k}(1) / 10 ;
            temper(k1,k2,k) = grid_optX{k1,k2,k}(2) / 10 ;
        end
    end
end

mem_k = mean(mem_k,3)
contourf(X,Y,mem_k) ; colorbar ; title('Memory noise') ;
xlabel('c-/c+ ratio') ; ylabel('c- cutoff') ;

%% Plot the temperature parameter
temper = mean(temper,3)
contourf(X,Y,temper) ; colorbar ; title('Softmax temperature') ;
xlabel('c-/c+ ratio') ; ylabel('c- cutoff') ;

%% Interim conclusion
% The optimum seems to be located for:
%  cutoff = .60   category_sz units
%  cutoff_ratio = 0.7
% Thus, cutoffs = .60 .* [-3 -1 .7 3*.7] = [-1.8 -.6 .42 1.26].
% Memory noise seems to be about mem_k = .050 (or less), which is less than
% the default (.070) in PetrovAnderson05.
% The temperature parameter seems poorly determined, but temper = .040
% seems a reasonable starting point for subsequent explorations. This is
% again less than the default (.050) in PetrovAnderson05.
optX = [.040 .050].*10 ;
Mparams = vector2params(optX,Sparams.params,Sparams.v2p_templ) ;
Mparams.cutoffs = .60 .* [-3 -1 .7 3*.7]

%% Generate and ARL2 profiles with these parameters
for k = 1:N_chevs
    Mdata{k}.params = Mparams ;
    opt_grARL2(:,:,k) = protoanchor_ARL2(Mdata{k}) ;
end
opt_grARL2 = mean(opt_grARL2,3) ;
[grARL2 opt_grARL2]
opt_SSE = (grARL2-opt_grARL2) ;
opt_SSE = sum(opt_SSE(:).^2)

%% Plot the "optimal" ARL2 profiles
ls = {'r-' 'b-' 'k<-' 'b--' 'k<--'} ;   % group line styles
lw = [1 2 2  2 2] ;                     % group line widths
els  = {'r:' 'b:' 'k<:' 'b:' 'k<:'} ;   % empirical-data line styles

x2 = [14 , 56:56:448]' ;
xtick = [28 , 140:112:476] ;
ytick = [3.6:.2:4.8] ;
ax = [0 476 3.6 4.8] ;

clf ; hold on ;
for k = 1:N_groups
    hh = plot(x2,opt_grARL2(:,k),ls{k}) ;
    set(hh,'LineWidth',lw(k),'MarkerSize',4) ;
end
for k = 1:N_groups
    plot(x2,grARL2(:,k),els{k}) ;
end
hold off ; box on ;
axis(ax) ; set(gca,'xtick',xtick,'ytick',ytick,'xgrid','on') ;
hh = xlabel('Trial') ; set(hh,'FontSize',14) ;
hh = ylabel('Average response level') ; set(hh,'FontSize',14) ;
hh = legend('U1','L1','H1','L2','H2',3) ; set(hh,'FontSize',14) ;
title(sprintf('INST fits, SSE = %.3f',opt_SSE)) ;

%% Define Assimilation := ARL(N)-ARL(P) and plot Assim profile
% See .../CMUExper/fdbk1/data/empir_ARL2_profile.m
%Assim = grARL2(:,[3 find(ugr==6)]) - grARL2(:,[2 find(ugr==5)])
opt_Assim = opt_grARL2(:,[3 find(ugr==6)]) - opt_grARL2(:,[2 find(ugr==5)])

ax = [0 476 -0.85 1.15] ;
ytick1 = [-.8:.2:1] ;
idx1 = [1 2 3 6 7] ;    % feedback blocks for the feedback-first groups
idx2 = [1 4 5 8 9] ;    % feedback blocks for the no-feedback-first grps

clf ;
hh = plot(x2,opt_Assim(:,1),'r-',x2,opt_Assim(:,2),'b--',...
          x2(idx1),opt_Assim(idx1,1),'ro',x2(idx2),opt_Assim(idx2,2),'bs') ;
    set(hh,'LineWidth',2) ;
    axis(ax) ;
    hh=refline(0,0) ; 
    set(hh,'Color','k','LineStyle','-','LineWidth',1) ;
box on ;
axis(ax) ; set(gca,'xtick',xtick,'ytick',ytick1,'xgrid','on') ;
hh = xlabel('Trial') ; set(hh,'FontSize',14) ;
hh = ylabel('ARL(H) - ARL(L)') ; set(hh,'FontSize',14) ;
hh = legend('Feedback first','No-feedback first',4) ; set(hh,'FontSize',14) ;

% The fitting effort continues in fit_INST_ARL2_part2.m

%% Clean up
clear k k1 k2 X Y ax hh ;
