function  params = protoanchor_params(spec)

% PROTOANCHOR_PARAMS --  Default parameters for the PROTOANCHOR model.
%   
%   PARAMS = PROTOANCHOR_PARAMS(seed_params)
%   PARAMS = PROTOANCHOR_PARAMS(avail)
%   PARAMS = PROTOANCHOR_PARAMS('LINEAR')
%   PARAMS = PROTOANCHOR_PARAMS('LOG')
%
%   SPEC is a string, number, or structure.
%   When it is a string, it specifies the SCALE parameter: either 'LINEAR'
%   (the default) or 'LOG'. When it is a vector, it specifies which 
%   instances are available initially (default [1 7]).
%   When it is a structure, it is taken as a seed that is augmented by
%   filling in default values to the slots that are not specified in the 
%   seed. Existing slot values are left intact.
%
%   Example: params = protoanchor_params
%     params =
%         scale: 'LINEAR'        % type of stim->mag conversion rule
%         N_cat: 7               % number of response categories
%       SM_conv: 0.001           % stim units  --> mag units, LINEAR
%        cat_sz: 0.0400          % mag units --> category units, LINEAR
%        perc_k: 0.040           % Ekman -- for power scales
%         mem_k: 0.070           % Ekman -- for power scales
%         avail: [1;7]           % only inst 1 and 7 available initially
%     instances: [Nx3 double]    % see DEFAULT_INSTANCES
%       cutoffs: [-2.4 -0.8 +0.72 +2.16]    % in category-size units
%        temper: 0.050           % in magnitude units
%       history: 0.100           % history factor
%         alpha: 0.30            % competitive-learning rate (not used)
%         decay: 0.50
%           ITI: 4               % inter-trial interval [sec]
%
%   See also ANCHOR_PARAMS, PROTOANCHOR, DEFAULT_INSTANCES.

% Original coding by Alex Petrov, Ohio State University
% $Revision: 1.0 $  $Date: 2007-04-06 15:16 $
%
% Copyright (c) Alexander Petrov 1999-2007, http://alexpetrov.com
% Please read the LICENSE and NO WARRANTY statement in anchor_license.m

%---  Decipher SPEC
if (nargin==0)                % default
  params = struct('scale','LINEAR') ;
  avail = [1;7] ;        % only anchors 1 and 7 initially available
elseif (ischar(spec))         % string
  params = struct('scale',upper(spec)) ;
  avail = [1;7] ;        % only anchors 1 and 7 initially available
elseif (isnumeric(spec))      % vector
  params = struct('scale','LINEAR') ;
  avail = spec(:) ;
elseif (isstruct(spec))       % structure
  params = spec ;             % copy existing slots
else
  error('The argument must be a string, number, or structure.') ;
end

%--- Fill-in missing slots
if (~isfield(params,'scale'))
  params.scale = 'LINEAR' ;      % Stevens' law with exponent n=1
else
  params.scale = upper(params.scale) ;
end
if (~isfield(params,'N_cat'))
  params.N_cat = 7 ;             % number of response categories
end
N_cat = params.N_cat ;

if (strncmp(params.scale,'LIN',3))      % LINEAR scale (default)
   LINscale = 1 ;
   dflt_SM_conv = 0.001 ;        % 475 pixels --> 0.475 magnitude units
   if (N_cat==7)
      anchor_loc = [420:40:660]' .* dflt_SM_conv ;  % optimal locations
      dflt_cat_sz = 40 * dflt_SM_conv ;
   else  % N_cat = 9
      anchor_loc = [275:50:675]' .* dflt_SM_conv ;
      dflt_cat_sz = 50 * dflt_SM_conv ;
   end
   dflt_perc_k = 0.040 ;         % Ekman's constant [in magnitude units]
   dflt_mem_k = 0.070 ;          % memory noise constant
   dflt_history = 2 * dflt_cat_sz ;     % activation units --> mag units
   dflt_temper = dflt_cat_sz ;
elseif (strncmp(params.scale,'LOG',3))  % LOG scale
   LINscale = 0 ;
   dflt_SM_conv = 0.475/log(475) ;      % = 0.0771 ; 475pix --> 0.475mags
   if (N_cat==7)
      anchor_loc = log([420:40:660]').*dflt_SM_conv ;
      dm = anchor_loc(2:7)-anchor_loc(1:6) ;
      dflt_cat_sz = geomean(dm) ;
   else  % N_cat = 9
      anchor_loc = log([275:50:675]').*dflt_SM_conv ;   %[.433 ... .502]
      dm = anchor_loc(2:9)-anchor_loc(1:8) ;
      dflt_cat_sz = geomean(dm) ;   % = 0.0084 ; average category width
   end
   dflt_perc_std = 0.040 ;          % j.n.d. <= log(1+k)/(z75*sqrt2) ~ k
   dflt_mem_std = 0.070 ;           % std.dev of the memory noise
   dflt_history = 2 * dflt_cat_sz ; % activation units --> mag units
   dflt_temper = dflt_cat_sz ;
else
   error('SCALE must be either ''LINEAR'' or ''LOG''.') ;
end

if (~isfield(params,'SM_conv'))
  params.SM_conv = dflt_SM_conv ;  % stimulus units  --> mag units
end
if (~isfield(params,'cat_sz'))
  params.cat_sz = dflt_cat_sz ;    % magnitude units --> cat-sz units
end
if (LINscale & ~isfield(params,'perc_k'))    % Ekman's perceptual const.
  params.perc_k = dflt_perc_k ;
end
if (LINscale & ~isfield(params,'mem_k'))     % Ekman's memory constant
   params.mem_k = dflt_mem_k ;
end
if (~LINscale & ~isfield(params,'perc_std')) % Thurstone's perc. std.dev.
  params.perc_std = dflt_perc_std ;          % times SM_conv
end
if (~LINscale & ~isfield(params,'mem_std'))  % Thurstone's memory std
   params.mem_std = dflt_mem_std ;           % times SM_conv
end

if (~isfield(params,'avail'))    % only inst1 & inst7 available initially
  params.avail = avail ;         % AVAIL supplied as an input argument
else
  avail = params.avail ;
end
if (~isfield(params,'instances'))
  instances = default_instances(anchor_loc([1 N_cat]),N_cat) ;
  instances(:,1) = anchor_loc ;    % optimal locations by default
  params.instances = instances(avail,:) ;
end
if (~isfield(params,'cutoffs'))  % for the correction mechanism
  params.cutoffs = [-3 -1 +0.9 +2.7] * 0.8 ;   % in category-size units
end
if (~isfield(params,'temper'))   % for the partial-matching mechanism
  params.temper = dflt_temper ;  % in magnitude units
end
if (~isfield(params,'history'))  % history factor (=1/mismatch_penalty)
  params.history = dflt_history ;  % activation units --> magnitude units
end
if (~isfield(params,'alpha'))    % competitive-learning rate (not used)
  params.alpha = 0.30 ;
end
if (~isfield(params,'decay'))    % of base-level activations
  params.decay = 0.50 ;
end
if (~isfield(params,'ITI'))
  params.ITI = 4 ;               % inter-trial interval [sec]
end

%- The fields below do not control aspects of the main model.
%- They are used for auxiliary purposes only.

%--- Return PARAMS
%%%%% End of file PROTOANCHOR_PARAMS.M
